@@ -280,7 +280,7 @@ class BaseClient(object):
             return False
 
         if r.status_code != codes.ok:
-            exc_cls, exc_msg = self._get_exception(r.headers)
+            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code)
             raise exc_cls(exc_msg)
 
         self.auth_token = r.headers['x-rucio-auth-token']
@@ -324,6 +324,8 @@ class BaseClient(object):
             try:
                 r = self.session.get(url, headers=headers, cert=cert, verify=self.ca_cert)
             except SSLError, e:
+                if 'alert certificate expired' in str(e.message):
+                    raise CannotAuthenticate(str(e))
                 LOG.warning('SSLError: ' + str(e))
                 self.ca_cert = False
                 retry += 1
@@ -337,7 +339,7 @@ class BaseClient(object):
             return False
 
         if r.status_code != codes.ok:
-            exc_cls, exc_msg = self._get_exception(r.headers)
+            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code)
             raise exc_cls(exc_msg)
 
         self.auth_token = r.headers['x-rucio-auth-token']
@@ -372,7 +374,7 @@ class BaseClient(object):
             return False
 
         if r.status_code != codes.ok:
-            exc_cls, exc_msg = self._get_exception(r.headers)
+            exc_cls, exc_msg = self._get_exception(headers=r.headers, status_code=r.status_code)
             raise exc_cls(exc_msg)
 
         self.auth_token = r.headers['x-rucio-auth-token']
