@@ -38,8 +38,8 @@ def validate_time_formats(time_string):
             err_msg = 'Parameter value [%s] is not a valid time delta !' % (time_string)
             raise InputValidationError(err_msg)
 
-            delta = timedelta(**dict([(key, (value and int(value) or 0)) for key, value in d.groupdict(0).items()]))
-            return delta
+        delta = timedelta(**dict([(key, (value and int(value) or 0)) for key, value in d.groupdict(0).items()]))
+        return delta
     except:
         err_msg = 'Parameter value [%s] is not a valid time delta !' % (time_string)
         raise InputValidationError(err_msg)
@@ -1016,7 +1016,7 @@ class DQ2Client:
             immutable = 0
         exists = False
         for rule in self.client.list_did_rules(scope, dsn):
-            if rule['rse_expression'] == location or [i['rse'] for i in self.client.list_rses(rule['rse_expression'])]:
+            if rule['rse_expression'] == location or location in [i['rse'] for i in self.client.list_rses(rule['rse_expression'])]:
                 exists = True
                 if rule['created_at'] < creationdate:
                     creationdate = rule['created_at']
@@ -1271,8 +1271,11 @@ class DQ2Client:
             dids = []
             did = {'scope': scope, 'name': dsn}
             dids.append(did)
-            # Add replication rule
-            self.client.add_replication_rule(dids=dids, copies=1, rse_expression=location, weight=None, lifetime=lifetime, grouping='NONE', account=None, locked=False)
+            # Check if a replication rule for scope:name, location, account already exists
+            for rule in self.client.list_did_rules(scope=scope, name=dsn):
+                if (rule['rse_expression'] == location) and (rule['account'] == self.client.account):
+                    return True
+            self.client.add_replication_rule(dids=dids, copies=1, rse_expression=location, weight=None, lifetime=lifetime, grouping='NONE', account=self.client.account, locked=False, notify='N')
         else:
             raise UnsupportedOperation('Unknown RSE [%s]' % (location))
         return True
@@ -1299,9 +1302,9 @@ class DQ2Client:
         @sshare: not used yet.
         @group: not used.
         @owner: not used yet.
-        @activity: not used.
+        @activity: is the activity.
         @acl_alias: not used.
-        @replica_lifetime: not used.
+        @replica_lifetime: is the replica lifetime.
         @check_destination: not used.
         @parentId: not used.
         @pin_lifetime: not used.
@@ -1322,16 +1325,27 @@ class DQ2Client:
                 raise InputValidationError(errMsg)
             pin_lifetime = pin_lifetime.days * 86400 + pin_lifetime.seconds
         if not owner:
-            owner = 'root'
+            owner = self.client.account
         else:
             l = [i for i in self.client.list_accounts('user', owner)]
             if l != []:
                 owner = l[0]['account']
         dids = [{'scope': scope, 'name': dsn}]
-        for rule in self.client.list_did_rules(scope=scope, name=dsn):
-            if (rule['rse_expression'] == location) and (rule['account'] == owner):
-                raise Duplicate('A rule for %s:%s at %s already exists' % (scope, dsn, location))
-        self.client.add_replication_rule(dids, copies=1, rse_expression=location, weight=None, lifetime=replica_lifetime, grouping='DATASET', account=owner, locked=False)
+
+        notify = 'N'
+        if callbacks != {}:
+            notify = 'C'
+
+        list_sources = sources.keys()
+        if len(list_sources) == 1 and list_sources[0] == location and location.find('TAPE') > -1:
+            # This is a staging request
+            attr = self.client.list_rse_attributes(location)
+            self.client.add_replication_rule(dids, copies=1, rse_expression=attr['staging_buffer'], weight=None, lifetime=replica_lifetime, grouping='DATASET', account=owner, locked=False, activity=activity, notify=notify)
+        else:
+            for rule in self.client.list_did_rules(scope=scope, name=dsn):
+                if (rule['rse_expression'] == location) and (rule['account'] == owner):
+                    raise Duplicate('A rule for %s:%s at %s already exists' % (scope, dsn, location))
+            self.client.add_replication_rule(dids, copies=1, rse_expression=location, weight=None, lifetime=replica_lifetime, grouping='DATASET', account=owner, locked=False, activity=activity, notify=notify)
 
     def registerDatasetsInContainer(self, name, datasets, scope=None):
         """
@@ -1457,16 +1471,13 @@ class DQ2Client:
             vuid = '%s-%s-%s-%s-%s' % (vuid[0:8], vuid[8:12], vuid[12:16], vuid[16:20], vuid[20:32])
             result[vuid] = None
             # get file information
-            lfns = []
-            guids = []
-            sizes = []
-            checksums = []
-            pfns = []
+            lfns, guids, sizes, checksums, pfns = [], [], [], [], []
             for file in datasets[dataset]:
                 guids.append(file['guid'])
                 sizes.append(file['size'])
                 checksums.append(file['checksum'])
-                pfns.append(file['surl'])
+                if 'surl' in file:
+                    pfns.append(file['surl'])
                 if 'scope' not in file:
                     if file['lfn'].find(':') > -1:
                         lfns.append(file['lfn'])
@@ -1534,7 +1545,7 @@ class DQ2Client:
         vuid = '%s-%s-%s-%s-%s' % (vuid[0:8], vuid[8:12], vuid[12:16], vuid[16:20], vuid[20:32])
         duid = vuid
         if rse:
-            self.client.add_replication_rule(dids=[{'scope': scope, 'name': dsn}], copies=1, rse_expression=rse, weight=None, lifetime=None, grouping='DATASET', account=None, locked=False)
+            self.client.add_replication_rule(dids=[{'scope': scope, 'name': dsn}], copies=1, rse_expression=rse, weight=None, lifetime=None, grouping='DATASET', account=self.client.account, locked=False, notify='N')
         return {'duid': duid, 'version': 1, 'vuid': vuid}
 
     def registerNewDataset2(self, dsn, lfns=[], guids=[], sizes=[], checksums=[], cooldown=None,  provenance=None, group=None, hidden=False, ignore=False, scope=None, rse=None):
@@ -1597,7 +1608,7 @@ class DQ2Client:
                     except FileAlreadyExists:
                         statuses[f['name']] = {'status': False, 'error': FileAlreadyExists, 'duid': duid}
         if rse:
-            self.client.add_replication_rule(dids=[{'scope': scope, 'name': dsn}], copies=1, rse_expression=rse, weight=None, lifetime=None, grouping='DATASET', account=None, locked=False)
+            self.client.add_replication_rule(dids=[{'scope': scope, 'name': dsn}], copies=1, rse_expression=rse, weight=None, lifetime=None, grouping='DATASET', account=self.client.account, locked=False, notify='N')
         return {'duid': duid, 'version': 1, 'vuid': vuid}, statuses
 
     def registerNewVersion(self, dsn, lfns=[], guids=[], sizes=[], checksums=[], ignore=False, scope=None):
@@ -1771,7 +1782,7 @@ class DQ2Client:
                         errMsg = 'lifetime must be greater than O!' % locals()
                         raise InputValidationError(errMsg)
                     lifetime = lifetime.days * 86400 + lifetime.seconds
-                    self.client.add_replication_rule(dids=[{'scope': scope, 'name': dsn}], copies=rule['copies'], rse_expression=location, weight=rule['weight'], lifetime=lifetime, grouping=rule['grouping'], account=account, locked=False)
+                    self.client.add_replication_rule(dids=[{'scope': scope, 'name': dsn}], copies=rule['copies'], rse_expression=location, weight=rule['weight'], lifetime=lifetime, grouping=rule['grouping'], account=account, locked=False, notify='N')
                     self.client.delete_replication_rule(rule['id'])
                 elif attrname == 'pin_lifetime':
                     if attrvalue is None or attrvalue is '':
@@ -1782,10 +1793,11 @@ class DQ2Client:
                             errMsg = 'pin_lifetime must be greater than O!' % locals()
                             raise InputValidationError(errMsg)
                         pin_lifetime = pin_lifetime.days * 86400 + pin_lifetime.seconds
-                        self.client.add_replication_rule(dids=[{'scope': scope, 'name': dsn}], copies=rule['copies'], rse_expression=location, weight=rule['weight'], lifetime=pin_lifetime, grouping=rule['grouping'], account=account, locked=True)
+                        self.client.add_replication_rule(dids=[{'scope': scope, 'name': dsn}], copies=rule['copies'], rse_expression=location, weight=rule['weight'],
+                                                         lifetime=pin_lifetime, grouping=rule['grouping'], account=account, locked=True, notify='N')
         if not is_at_site:
             raise UnsupportedOperation('Replicas for %s:%s at %s does not exist' % (scope, dsn, location))
-        self.client.add_replication_rule(dids=[{'scope': scope, 'name': dsn}], copies=1, rse_expression=location, weight=None, lifetime=lifetime, grouping='DATASET', account=account, locked=False)
+        self.client.add_replication_rule(dids=[{'scope': scope, 'name': dsn}], copies=1, rse_expression=location, weight=None, lifetime=lifetime, grouping='DATASET', account=account, locked=False, notify='N')
         return 0
 
     def verifyFilesInDataset(self, dsn, guids, version=None, scope=None):
