@@ -9,6 +9,7 @@
 # - Ralph Vigne, <ralph.vigne@cern.ch>, 2012-2014
 # - Cedric Serfon, <cedric.serfon@cern.ch>, 2013-2014
 # - Wen Guan, <wen.guan@cern.ch>, 2014
+# - Cheng-Hsi Chao, <cheng-hsi.chao@cern.ch>, 2014
 
 import hashlib
 
@@ -40,6 +41,8 @@ class RSEProtocol(object):
                 setattr(self, 'lfns2pfns', self.__lfns2pfns_client)
             if rsemanager.SERVER_MODE:
                 setattr(self, '_get_path', self._get_path_nondeterministic_server)
+        else:
+            self.attributes['determinism_type'] = 'default'
 
     def lfns2pfns(self, lfns):
         """
@@ -60,10 +63,24 @@ class RSEProtocol(object):
         lfns = [lfns] if type(lfns) == dict else lfns
         for lfn in lfns:
             scope, name = lfn['scope'], lfn['name']
-            if lfn['path'] is not None:
-                pfns['%s:%s' % (scope, name)] = ''.join([self.attributes['scheme'], '://', self.attributes['hostname'], ':', str(self.attributes['port']), prefix, lfn['path']])
+            if 'path' in lfn and lfn['path'] is not None:
+                pfns['%s:%s' % (scope, name)] = ''.join([self.attributes['scheme'],
+                                                         '://',
+                                                         self.attributes['hostname'],
+                                                         ':',
+                                                         str(self.attributes['port']),
+                                                         prefix,
+                                                         lfn['path'] if not lfn['path'].startswith('/') else lfn['path'][1:]
+                                                         ])
             else:
-                pfns['%s:%s' % (scope, name)] = ''.join([self.attributes['scheme'], '://', self.attributes['hostname'], ':', str(self.attributes['port']), prefix, self._get_path(scope=scope, name=name)])
+                pfns['%s:%s' % (scope, name)] = ''.join([self.attributes['scheme'],
+                                                         '://',
+                                                         self.attributes['hostname'],
+                                                         ':',
+                                                         str(self.attributes['port']),
+                                                         prefix,
+                                                         self._get_path(scope=scope, name=name)
+                                                         ])
         return pfns
 
     def __lfns2pfns_client(self, lfns):
@@ -133,6 +150,8 @@ class RSEProtocol(object):
             scheme = parsed.scheme
             hostname = parsed.netloc.partition(':')[0]
             port = int(parsed.netloc.partition(':')[2]) if parsed.netloc.partition(':')[2] != '' else 0
+            while '//' in parsed.path:
+                parsed = parsed._replace(path=parsed.path.replace('//', '/'))
             path = parsed.path
 
             # Protect against 'lazy' defined prefixes for RSEs in the repository
